#ifndef AINetworkArc_h
#define AINetworkArc_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Map/MapObject.h>

#include <Airport/Airport.h>
#include "AINetworkVertex.h"

class AINetworkArc: public MapObject {
        
        DECLARE_CLASS(AINetworkArc)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(AINetworkArc);
        
public:
        AINetworkArc(TransactionManager* transactionManager,
                     AINetworkVertex::SmartRef source,
                     AINetworkVertex::SmartRef target,
                     bool isBidirectional,
                     bool isPushBackRoute,
                     wxString name=wxT(""));
        
        virtual ~AINetworkArc();
        
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        void PaintArc(wxDC& dc, MapEditor* editor, wxColour colour);
        
        virtual bool Hit(MapEditor* editor, TDRectangle r);
        
        virtual MapSelection* CreateSelection(MapSelectionManager* manager);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        virtual TDRectangle GetGeographicBounds();
        
        virtual void OnDetach();
        
        AINetworkArc::SmartRef SplitAtCenter(MapEditor* editor);
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(AINetworkVertex::SmartRef,Source,source)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(AINetworkVertex::SmartRef,Target,target)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(bool,IsBidirectional,isBidirectional)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(bool,IsPushBackRoute,isPushBackRoute)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(wxString,Name,name,
                                                TRANSACTIONOBJECT_STATEVAR(name).Cmp(v)==0)
        
        bool IsBidirectional() const {
                return GetIsBidirectional();
        }
        
        bool IsPushBackRoute() const {
                return GetIsPushBackRoute();
        }
        
        static double Arc_Width;
protected:
        typedef struct {
                AINetworkVertex::SmartRef source,target;
                bool isBidirectional;
                bool isPushBackRoute;
                wxString name;
        } State;
        
        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(AINetworkArc,MapObject,State)
protected:
        void GetArrowPoints(MapEditor* editor, Point3D pts[3]);
};

#endif

