#include "AINetworkNodePlacementMode.h"

#include "AINetworkArc.h"
#include "AINetworkNode.h"
#include "AINetworkVertexSelection.h"

#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(AINetworkNodePlacementMode,Mode)

BEGIN_EVENT_TABLE(AINetworkNodePlacementMode,Mode)
        EVT_LEFT_DOWN(AINetworkNodePlacementMode::OnLeftDown)
END_EVENT_TABLE()

AINetworkNodePlacementMode::AINetworkNodePlacementMode(Airport::SmartRef airport,
                                                       bool connect,
                                                       bool bidirectional):
        _airport(airport),_connect(connect),_bidirectional(bidirectional)
{
}

AINetworkNodePlacementMode::~AINetworkNodePlacementMode() {
}

/* Get a string for the status bar. */
wxString AINetworkNodePlacementMode::GetStatus() {
        return wxT("Click to position a new network node");
}

void AINetworkNodePlacementMode::OnLeftDown(wxMouseEvent& evt) {
        if (evt.AltDown() || evt.ControlDown() || evt.ShiftDown()) {
                evt.Skip();
                return;
        }
        
        MapEditor* editor=GetModeManager()->GetEditor();
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        MapProjection* proj=editor->GetProjection();
        MapSelectionManager& selmgr=editor->GetSelectionManager();
        
        Point3D localPos=cs.DeviceToLogical(evt.GetPosition());
        Point3D geoPos=proj->ConvertFromLocal(localPos);
        
        AINetworkVertex::SmartRef node=new AINetworkNode(_airport->GetTransactionManager(),geoPos);
        _airport->GetGroundNetworkLayer()->AttachObject(node);
        
        if (_connect) {
                MapSelectionList list;
                selmgr.GetSelectionsOfType(list, CLASSINFO(AINetworkVertexSelection));    
                MapSelectionList::Node* listNode;
                
                for (listNode=list.GetFirst();listNode;listNode=listNode->GetNext()) {
                        AINetworkVertexSelection* sel=(AINetworkVertexSelection*)listNode->GetData();
                        AINetworkVertex::SmartRef vertex=sel->GetContent();
                        AINetworkArc::SmartRef arc=new AINetworkArc(_airport->GetTransactionManager(),
                                vertex,node,_bidirectional,false);
                        _airport->GetGroundNetworkLayer()->AttachObject(arc);
                }
                
                selmgr.DeselectAll();
                selmgr.AddSelectionFor(node);
        }
        
        editor->GetTransactionManager()->Commit(wxT("AI Node Placement"));
}
