#ifndef AINetworkVertex_h
#define AINetworkVertex_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Map/MapObject.h>

#include <Airport/Airport.h>

class AINetworkArc;
WX_DECLARE_LIST(AINetworkArc,AINetworkVertexArcs);

class AINetworkVertex: public MapObject {
        
        DECLARE_CLASS(AINetworkVertex)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(AINetworkVertex);
        
public:
        AINetworkVertex(TransactionManager* transactionManager,
                        Point3D position);
        
        virtual ~AINetworkVertex();
        
        void AttachArc(AINetworkArc* arc);
        
        void DetachArc(AINetworkArc* arc);
        
        void MergeArcsFrom(AINetworkVertex::SmartRef v);
        
        virtual void OnDetach();
        
        virtual void BeforeWrite();
        
        virtual void AfterWrite();
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(Point3D,Position,position)
        
        static const MapPass AINetworkPass;

        void setPathScore(double val) {  pathScore = val; }
        void setPreviousNode(AINetworkVertex * prev) { previousVertex = prev; };
        void setPreviousArc (AINetworkArc    * prev) { previousArc    = prev; };

        double getPathScore() { return pathScore; };
        AINetworkVertex *getPreviousNode() { return previousVertex; };

protected:
        typedef struct {
                Point3D position;
                AINetworkVertexArcs arcs;
        } State;
        
        TRANSACTIONOBJECT_DEFINE_STATE(State)
        
        void DamageArcs();
        
        Airport::SmartRef _airport;

       // Used in path finding
       double pathScore;
       AINetworkVertex *previousVertex;
       AINetworkArc    *previousArc;
protected:
        class Memento: public MapObject::Memento {
                friend class AINetworkVertex;
        public:
                Memento(const AINetworkVertex& vertex);
                
                virtual ~Memento();
        protected:
                State _savedState;
                MapObjectList _objects;
        };
public:
        wxAINetworkVertexArcsNode *getFirstArc();
        virtual Memento* CreateMemento() {
                return new Memento(*this);
        }
        
        virtual void RestoreMemento(wxObject* object);
};

#endif

