#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "PointObjectSelection.h"
#include "PointMapObject.h"
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(PointObjectSelection,MapSelection);

PointObjectSelection::PointObjectSelection(MapSelectionManager* manager, PointMapObject* content):
        MapSelection(manager,content) {
}

PointObjectSelection::~PointObjectSelection() {
}

void PointObjectSelection::DeltaDrag(Point3D delta) {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapProjection* proj=editor->GetProjection();
        
        Point3D pos=GetContent()->GetPosition();
        Point3D localPos=proj->ConvertToLocal(pos);
        localPos+=delta;
        pos=proj->ConvertFromLocal(localPos);
        
        GetContent()->SetPosition(pos);
}

void PointObjectSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(PointObjectSelection)))
                return;
        propgrid->Append(wxFloatProperty(wxT("Latitude (deg)"),wxT("PointObjectSelection_Lat")));
        propgrid->Append(wxFloatProperty(wxT("Longitude (deg)"),wxT("PointObjectSelection_Lon")));
}

void PointObjectSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(PointObjectSelection)))
                return;
        Point3D center=GetContent()->GetPosition();
        propgrid->SetPropertyValue(wxT("PointObjectSelection_Lat"),center.lat());
        propgrid->SetPropertyValue(wxT("PointObjectSelection_Lon"),center.lon());
}

void PointObjectSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        MapSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(PointObjectSelection)))
                return;
        PointMapObject* myContent=GetContent();
        PointMapObject* firstContent=wxDynamicCast(firstSelection,PointObjectSelection)->GetContent();
        if (myContent->GetPosition().lat()!=firstContent->GetPosition().lat())
                propgrid->SetPropertyUnspecified(wxT("PointObjectSelection_Lat"));
        if (myContent->GetPosition().lon()!=firstContent->GetPosition().lon())
                propgrid->SetPropertyUnspecified(wxT("PointObjectSelection_Lon"));
}

void PointObjectSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString name=property->GetName();
        wxVariant value=property->GetValueAsVariant();
        if (name.Cmp(wxT("PointObjectSelection_Lat"))==0) {
                Point3D center=GetContent()->GetPosition();
                center.setlat(value.GetDouble());
                GetContent()->SetPosition(center);
        } else if (name.Cmp(wxT("PointObjectSelection_Lon"))==0) {
                Point3D center=GetContent()->GetPosition();
                center.setlon(value.GetDouble());
                GetContent()->SetPosition(center);
        } else {
                MapSelection::UpdateObjectFromProperty(property);
        }
}

