#ifndef RunwayMapObject_h
#define RunwayMapObject_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "PavementMapObject.h"

/**
 * \brief A RunwayMapObject represents a runway.
 */
class RunwayMapObject: public PavementMapObject {
        
        DECLARE_CLASS(RunwayMapObject)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(RunwayMapObject);
        
public:
        typedef enum {
                VAPL_None,
                VAPL_VASI,
                VAPL_PAPI,
                VAPL_SpaceShuttlePAPI
        } VisualApproachPathLighting;
        
        typedef enum {
                RWYL_None,
                RWYL_EdgeLights,
                RWYL_REIL,
                RWYL_CLL,
                RWYL_TDZ
        } RunwayLighting;
        
        typedef enum {
                APPL_None,
                APPL_SSALS,
                APPL_SALSF,
                APPL_ALSF_I,
                APPL_ALSF_II,
                APPL_ODALS,
                APPL_Calvert_1,
                APPL_Calvert_2_3
        } ApproachLighting;
        
        typedef struct {
                int displacedThreshold;
                int stopwayLength;
                float vasiAngle;
                VisualApproachPathLighting visualApproachPathLighting;
                RunwayLighting runwayLighting;
                ApproachLighting approachLighting;
        } RunwayEndData;
        
        typedef enum {
                Shoulder_None,
                Shoulder_Asphalt,
                Shoulder_Concrete
        } Shoulder;
        
        typedef enum {
                Markings_None,
                Markings_Visual,
                Markings_NonPrecision,
                Markings_Precision
        } Markings;

	RunwayMapObject(Airport::SmartRef airport,
                        wxString designation,
                        Point3D center,
                        double length,
                        double width,
                        double heading,
                        Surface surface,
                        float smoothness,
                        RunwayEndData designatedEnd,
                        RunwayEndData oppositeEnd,
                        Markings markings,
                        Shoulder shoulder,
                        bool hasDistanceRemainingSigns);
        
        /* May only be called by MapLayer */
        virtual ~RunwayMapObject();
        
        /* Paint this object using the given device context into the given editor. */
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        virtual bool Hit(MapEditor* editor, TDRectangle r);

        virtual MapSelection* CreateSelection(MapSelectionManager* manager);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        /* Return the geographic (lat/lon) bounds */
        virtual TDRectangle GetGeographicBounds();

        wxString GetOppositeDesignation() const;
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WUCOMP(wxString,Designation,_designation,
                                               GetDesignation().Cmp(v)==0)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(RunwayEndData,DesignatedEnd,_designatedEnd)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(RunwayEndData,OppositeEnd,_oppositeEnd)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(Shoulder,Shoulder,_shoulder)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(Markings,Markings,_markings)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(bool,DistanceRemainingSigns,_hasDistanceRemainingSigns)
        
        bool HasDistanceRemainingSigns() const { return GetDistanceRemainingSigns(); }
        virtual const Point3D* GetEndCorners() const { return _stopwayCorners; }
        virtual const Point3D* GetEndCenters() const { return _stopwayCenterEnds; }
        virtual const Point3D* GetStopwayCorners() const { return PavementMapObject::GetEndCorners(); }
        virtual const Point3D* GetStopwayCenters() const { return PavementMapObject::GetEndCenters(); }
        virtual const Point3D* GetThresholdCorners() const { return _dispThreshCorners; }
        virtual const Point3D* GetThresholdCenters() const { return _dispThreshCenterEnds; }
protected:
        virtual void UpdateCorners();
        
        Point3D _dispThreshCenterEnds[2];
        Point3D _dispThreshCorners[4];
        Point3D _stopwayCenterEnds[2];
        Point3D _stopwayCorners[4];
        
        typedef struct {
                wxString _designation;
                RunwayEndData _designatedEnd,_oppositeEnd;
                Shoulder _shoulder;
                Markings _markings;
                bool _hasDistanceRemainingSigns;
        } State;
        
        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(RunwayMapObject,PavementMapObject,State)
};

#endif

