#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "TowerSelection.h"
#include "TowerMapObject.h"
#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(TowerSelection,PointObjectSelection);

TowerSelection::TowerSelection(MapSelectionManager* manager, TowerMapObject* content):
        PointObjectSelection(manager,content) {
}

TowerSelection::~TowerSelection() {
}

void TowerSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PointObjectSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(TowerSelection)))
                return;
        
        propgrid->Append(wxFloatProperty(wxT("Height"), wxT("TowerSelection_Height")));
        propgrid->Append(wxBoolProperty(wxT("Draw Tower"), wxT("TowerSelection_DrawTower")));
        propgrid->Append(wxStringProperty(wxT("Name"), wxT("TowerSelection_Name")));
}

void TowerSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PointObjectSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(TowerSelection)))
                return;

        propgrid->SetPropertyValue(wxT("TowerSelection_Height"),GetContent()->GetHeight());
        propgrid->SetPropertyValue(wxT("TowerSelection_DrawTower"),GetContent()->GetDrawTower());
        propgrid->SetPropertyValue(wxT("TowerSelection_Name"),GetContent()->GetName());
}

void TowerSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        PointObjectSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(TowerSelection)))
                return;
        TowerMapObject* firstContent=wxDynamicCast(firstSelection,TowerSelection)->GetContent();
        TowerMapObject* myContent=GetContent();

        if (firstContent->GetHeight()!=myContent->GetHeight())
                propgrid->SetPropertyUnspecified(wxT("TowerSelection_Height"));
        if (firstContent->GetDrawTower()!=myContent->GetDrawTower())
                propgrid->SetPropertyUnspecified(wxT("TowerSelection_DrawTower"));
        if (firstContent->GetName().Cmp(myContent->GetName())!=0)
                propgrid->SetPropertyUnspecified(wxT("TowerSelection_Name"));
}

void TowerSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("TowerSelection_Height"))==0) {
                GetContent()->SetHeight(value.GetDouble());
        } else if (name.Cmp(wxT("TowerSelection_DrawTower"))==0) {
                GetContent()->SetDrawTower(value.GetBool());
        } else if (name.Cmp(wxT("TowerSelection_Name"))==0) {
                GetContent()->SetName(property->GetValueAsString());
        } else {
                PointObjectSelection::UpdateObjectFromProperty(property);
        }
}


