#include "XMLProjectWriter.h"

IMPLEMENT_CLASS(XMLProjectWriter,wxObject);

XMLProjectWriter::XMLProjectWriter(Airport::SmartRef airport):
        wxObject(),
        _airport(airport)
{
}

XMLProjectWriter::~XMLProjectWriter() {
}

void XMLProjectWriter::writeProject(std::ostream& outfile) {
        outfile << "<?xml version=\"1.0\"?>\n";
        outfile << "<project>\n";
        void writeAirportDefinition(std::ostream& outfile);
        void writeRunways(std::ostream& outfile);
        void writeTaxiways(std::ostream& outfile);
        void writeObjects(std::ostream& outfile);
        void writeGroundNetwork(std::ostream& outfile);
        outfile << "</project>\n";
}

void XMLProjectWriter::writeAirportDefinition(std::ostream& outfile) {
        std::string apttype;
        
        switch (_airport->GetType()) {
        case Airport::AirportType_Land:
                apttype=std::string("land");
                break;
        case Airport::AirportType_Sea:
                apttype=std::string("sea");
                break;
        case Airport::AirportType_Heliport:
                apttype=std::string("heliport");
                break;
        }
        outfile << "  <airport icao=\"" << (const char*)_airport->GetICAOID().mb_str(wxConvUTF8) << "\"\n"
                << "           name=\"" << (const char*)_airport->GetName().mb_str(wxConvUTF8) << "\"\n"
                << "           type=\"" << apttype << "\"\n"
                << "           hasControlTower=\"" << _airport->HasControlTower() << "\"\n"
                << "           hasDefaultBuildings=\"" << _airport->HasDefaultBuildings() << "\">\n";
        
        writeFrequencies(outfile);
        
        outfile << "  </airport>\n";
}

void XMLProjectWriter::writeFrequencies(std::ostream& outfile) {
        const FrequencyEntryArray& frequencies=_airport->GetFrequencies();
        for (size_t i=0;i<frequencies.GetCount();i++) {
                FrequencyEntry frequency=frequencies.Item(i);
                std::string freqtype;
                
                switch (frequency.GetType()) {
                case FrequencyEntry::FrequencyType_AWOS:
                        freqtype="AWOS";
                        break;
                case FrequencyEntry::FrequencyType_UNICOM:
                        freqtype="UNICOM";
                        break;
                case FrequencyEntry::FrequencyType_Clearance:
                        freqtype="Clearance";
                        break;
                case FrequencyEntry::FrequencyType_Ground:
                        freqtype="Ground";
                        break;
                case FrequencyEntry::FrequencyType_Tower:
                        freqtype="Tower";
                        break;
                case FrequencyEntry::FrequencyType_Approach:
                        freqtype="Approach";
                        break;
                case FrequencyEntry::FrequencyType_Departure:
                        freqtype="Departure";
                        break;
                }
                
                outfile << "    <frequency type=\"" << freqtype << "\""
                        <<               " frequency=\"" << (const char*)frequency.GetFrequency().mb_str(wxConvUTF8) << "\""
                        <<               " name=\"" << (const char*)frequency.GetName().mb_str(wxConvUTF8) << "\" />";
        }
}

void XMLProjectWriter::writeRunways(std::ostream& outfile) {
}

void XMLProjectWriter::writeTaxiways(std::ostream& outfile) {
}

void XMLProjectWriter::writeObjects(std::ostream& outfile) {
}

void XMLProjectWriter::writeGroundNetwork(std::ostream& outfile) {
}

