#ifndef XPlaneLoader_h
#define XPlaneLoader_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Airport/Airport.h>
#include <Airport/PavementMapObject.h>
#include <Airport/RunwayMapObject.h>

#include <iostream>
#include <string>

class TransactionManager;
class FrequencyEntryArray;

class XPlaneLoader: public wxObject {
        
        DECLARE_CLASS(XPlaneLoader)
        
public:
	XPlaneLoader(TransactionManager* tmgr, std::istream& infile);
        
        void setVersion(int version) {
                _version=version;
        }
        
        virtual ~XPlaneLoader();
        
        Airport::SmartRef loadAirport(const std::string& icao);
        Airport::SmartRef scanAirport(char* header);
protected:
        
        void scanTowerLocationLine(Airport::SmartRef airport,char* line);
        void scanStartupLocationLine(Airport::SmartRef airport,char* line);
        void scanBeaconLine(Airport::SmartRef airport,char* line);
        void scanWindsockLine(Airport::SmartRef airport,char* line);
        void scanFrequencyLine(Airport::SmartRef airport,char* buf, FrequencyEntryArray& entries);
        void scanPavementLine(Airport::SmartRef airport,char* line);
        
        RunwayMapObject::VisualApproachPathLighting GetVAPLFromCode(int code);
        RunwayMapObject::RunwayLighting GetRWYLFromCode(int code);
        RunwayMapObject::ApproachLighting GetAPPLFromCode(int code);

        PavementMapObject::Surface GetSurfaceFromCode(int code);

        RunwayMapObject::Shoulder GetShoulderFromCode(int code);
        RunwayMapObject::Markings GetMarkingsFromCode(int code);
        
        TransactionManager* _transactionManager;
        std::istream *_infile;
        
        int _version;
};

#endif

