#include "MapSimpleBackgroundImageCornerHandle.h"

#include <Map/MapEditor.h>
#include <Map/MapProjection.h>

IMPLEMENT_CLASS(MapSimpleBackgroundImageCornerHandle,MapSelectionHandle);

MapSimpleBackgroundImageCornerHandle::MapSimpleBackgroundImageCornerHandle(
                MapSimpleBackgroundImageSelection* selection,
                unsigned int corner):
        MapSelectionHandle(selection),
        _corner(corner)
{
        MapEditor* editor=GetSelection()->GetEditor();
        GetSelection()->GetContent()->GetLocalCorners(editor,_localCorners);
        _handleOrigin=_localCorners[_corner];
        _originalScale=GetSelection()->GetContent()->GetScale();
        // determine horizontal, vertical and diagonal direction vectors
        _horizontal=(_localCorners[1]-_localCorners[0])/_originalScale;
        _vertical=(_localCorners[3]-_localCorners[0])/_originalScale;
        _diagonal=(_localCorners[_corner^2]-_localCorners[_corner])/_originalScale;
        _diagonalLengthSquared=_diagonal.distance3Dsquared(Point3D(0.0,0.0,0.0));
}

MapSimpleBackgroundImageCornerHandle::~MapSimpleBackgroundImageCornerHandle()
{
}

void MapSimpleBackgroundImageCornerHandle::StartDrag(Point3D currentPosition) {
        _dragOrigin=currentPosition;
}

void MapSimpleBackgroundImageCornerHandle::Drag(Point3D currentPosition) {
        Point3D delta=currentPosition-_dragOrigin;
        double d=(delta.x()*_diagonal.x()+delta.y()*_diagonal.y())/_diagonalLengthSquared;
        Point3D localNWPos=_localCorners[0];
        double scale;
        
        
        // clip this to scale down at most to 0.0 and not move the edge further
        if (d>_originalScale) {
                d=_originalScale;
        }
        scale=_originalScale-d;
        
        /*
         * adapt the position of the north-west point
         * corner 0 is north-west, numbering is in clockwise order
         *
         * The basic equations are:
         * x0'=x0+dx0
         * x1'=x1+dx1
         * x2'=x2+dx2
         * x3'=x3+dx3
         *
         * with dxi=0 for i=_corner xor 2.
         *
         * We express the points in terms of s and s' (old and new scale),
         * x0 and x0' (old and new northwest position) and x and y (horizontal
         * resp. vertical vector with |x|=w and |y|=h, the width and height of
         * the original image in pixels).
         *
         * x0'=x0+dx0
         * x0'+s'*x     =x0+s*x+dx1
         * x0'+s'*x+s'*y=x0+s*x+s*y+dx2
         * x0'     +s'*y=x0+    s*y+dx3
         *
         * This is only solvable if dxj with j=_corner is parallel to the
         * diagonal xi-xj. Then s'=s-d with dxj=d*(xi-xj).
         *
         * From this we also get the different dx00.
         */
        if (_corner==0) {
                localNWPos+=d*_diagonal;
        } else if (_corner==1) {
                localNWPos+=d*_vertical;
        } else if (_corner==3) {
                localNWPos+=d*_horizontal;
        }
        
        MapProjection* proj=GetSelection()->GetEditor()->GetProjection();
        Point3D geoNWPos=proj->ConvertFromLocal(localNWPos);
        GetSelection()->GetContent()->SetNorthWestPos(geoNWPos);
        GetSelection()->GetContent()->SetScale(scale);
}

wxString MapSimpleBackgroundImageCornerHandle::EndDrag() {
        return wxT("Changing image scale and position");
}

