#ifndef MapFrame_h
#define MapFrame_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Airport/Airport.h>
#include <Imagery/MapUSGSImageLayer.h>
#include <Imagery/USGSProgressListener.h>
#include <Map/MapEditor.h>
#include <Map/MapObjectContainer.h>
#include <Map/ModeChangeListener.h>
#include <Transactions/TransactionStackChangeListener.h>

#include <wx/menutoolbar/menutoolbar.h>

class MapStatusBar;
class MapPropertiesFrame;
class UndoRedoMenuManager;

class MapFrame: public wxFrame,
                public ModeChangeListener,
                public TransactionStackChangeListener,
                public USGSProgressListener
{
        
        DECLARE_CLASS(MapFrame)
        
public:
	MapFrame(Airport* airport,
                 MapObjectContainer* backgroundImageLayer=NULL,
                 wxWindow* parent=NULL,
                 wxWindowID id=-1,
                 const wxString& title=wxEmptyString,
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxSize(800,600),
                 long style = wxDEFAULT_FRAME_STYLE,
                 const wxString& name = wxT("mapframe"));
        
        virtual ~MapFrame();
        
        wxToolBar* OnCreateToolBar(long style, wxWindowID id, const wxString& name);
        
        const Airport::SmartRef& GetAirport() const { return _airport; }
        
        MapEditor*GetEditor() const { return _mapEditor; }
        
        void SetProjectFilename(const wxString& name) { _projectFilename=name; }
        
        void OnMenuOpen(wxMenuEvent& evt);
        
        void OnNewProject(wxCommandEvent& evt);
        void OnOpenProject(wxCommandEvent& evt);
        void OnSaveProject(wxCommandEvent& evt);
        bool DoSaveProject(bool saveAs);
        void OnImportAirport(wxCommandEvent& evt);
        void OnExportAirport(wxCommandEvent& evt);
        void OnImportAINetwork(wxCommandEvent& evt);
        void OnExportAINetwork(wxCommandEvent& evt);

        void OnClose(wxCommandEvent& evt);
        void OnExit(wxCommandEvent& evt);

        void OnUndo(wxCommandEvent& evt);
        void OnRedo(wxCommandEvent& evt);
        void OnDelete(wxCommandEvent& evt);
        void OnDeselectAll(wxCommandEvent& evt);
        void OnPreferences(wxCommandEvent& evt);

        void OnNewView(wxCommandEvent& evt);
        void OnShowProperties(wxCommandEvent& evt);
        void OnToggleGrid(wxCommandEvent& evt);
        void OnZoomIn(wxCommandEvent& evt);
        void OnZoomOut(wxCommandEvent& evt);
        void OnZoomReset(wxCommandEvent& evt);
        void OnZoomFullView(wxCommandEvent& evt);
        void OnZoomToSelection(wxCommandEvent& evt);
        void OnScroll(wxCommandEvent& evt);
        void OnLoadBackgroundImage(wxCommandEvent& evt);

        void OnVerifyGroundNetwork(wxCommandEvent& evt);
        
        void OnMode(wxCommandEvent& evt);
        void ActivateMode(int id);
        virtual void BeforeEnteringMode(Mode* mode);
        virtual void AfterLeavingMode(Mode* mode);
        
        void OnImageryThemeType(wxCommandEvent& evt);
        void OnImageryResolution(wxCommandEvent& evt);
        
        void OnFlipObjectsOrder(wxCommandEvent& evt);
        
        void OnKeyDown(wxKeyEvent& evt);
        
        void OnFillUndoMenu(wxCommandEvent& evt);
        void OnFillRedoMenu(wxCommandEvent& evt);
        
        virtual void OnTransactionStackChange(TransactionManager* tmgr);
        
        virtual void UpdateProgress(unsigned int pending, unsigned int total);
        
        void ActivateView();
        void DeactivateView();
        
        void OnIconize(wxIconizeEvent& evt);
        
        void OnActivate(wxActivateEvent& evt);
        
        void OnCloseWindow(wxCloseEvent& evt);
protected:
        DECLARE_EVENT_TABLE()
        
        void SetupMenubar();
        void SetupToolbar();
        void SetupDrawingWindow();
        void SetupMapEditor();
        
        void DisconnectMenuEvents(wxMenu* menu);
        
        void LoadAINetwork(wxString path);
        void SaveAINetwork(wxString path);
        
        Airport::SmartRef _airport;
        
        MapEditor *_mapEditor;
        MapPropertiesFrame* _propertyFrame;
        wxPanel *_drawingWindow;
        
        wxScrollBar *_vertScrollbar,*_horizScrollbar;
        
        wxMenuToolBar* _toolbar;
        MapStatusBar* _statusBar;
        
        wxMenu *_fileMenu,*_editMenu,*_actionsMenu,*_viewMenu,*_toolsMenu,*_projectionsMenu;
        UndoRedoMenuManager *_undoMenuManager,*_redoMenuManager;
        
        MapObjectContainer::SmartRef _backgroundImageLayer;
        MapUSGSImageLayer::SmartRef _usgsImageLayer;
        
        wxString _projectFilename;
        
        bool _showPropertyFrameOnActivation;
};

#endif
// :mode=c++:
