#include "PreferenceDialog.h"

#include <AI/AINetworkArc.h>
#include <AI/AINetworkNode.h>
#include <Map/MapEditor.h>
#include <Map/MapSelection.h>
#include <Modes/SelectionMode.h>

#include <wx/colordlg.h>
#include <wx/config.h>
#include <wx/filename.h>
#include <wx/notebook.h>

#if !wxHAS_TEXT_WINDOW_STREAM
#include <sstream>
#endif

IMPLEMENT_CLASS(PreferenceDialog,wxPropertySheetDialog);

enum {
        PreferenceDialog_ID_MapBackgroundColourButton=wxID_HIGHEST,
        PreferenceDialog_ID_SelectionHandleColourButton,
        PreferenceDialog_ID_SelectionMarkingColourButton,
        PreferenceDialog_ID_XPlaneImportPath,
        PreferenceDialog_ID_XPlaneImportPathBrowse,
        PreferenceDialog_ID_USGSCachePath,
        PreferenceDialog_ID_USGSCachePathBrowse,
        PreferenceDialog_ID_UseProxy,
        PreferenceDialog_ID_ProxyName,
        PreferenceDialog_ID_ProxyPort,
        PreferenceDialog_ID_FGFSDataPath,
        PreferenceDialog_ID_FGFSDataPathBrowse,
        PreferenceDialog_ID_FGFSSceneryPath,
        PreferenceDialog_ID_FGFSSceneryPathBrowse
};

BEGIN_EVENT_TABLE(PreferenceDialog,wxPropertySheetDialog)
        EVT_BUTTON(PreferenceDialog_ID_MapBackgroundColourButton, PreferenceDialog::OnColourBrowse)
        EVT_BUTTON(PreferenceDialog_ID_SelectionHandleColourButton, PreferenceDialog::OnColourBrowse)
        EVT_BUTTON(PreferenceDialog_ID_SelectionMarkingColourButton, PreferenceDialog::OnColourBrowse)
        EVT_CHECKBOX(PreferenceDialog_ID_UseProxy, PreferenceDialog::OnUseProxy)
        EVT_BUTTON(PreferenceDialog_ID_XPlaneImportPathBrowse, PreferenceDialog::OnXPlanePathBrowse)
        EVT_BUTTON(PreferenceDialog_ID_USGSCachePathBrowse, PreferenceDialog::OnUSGSImagePathBrowse)
        EVT_BUTTON(PreferenceDialog_ID_FGFSDataPathBrowse, PreferenceDialog::OnFGFSPathBrowse)
        EVT_BUTTON(PreferenceDialog_ID_FGFSSceneryPathBrowse, PreferenceDialog::OnFGFSSceneryBrowse)
END_EVENT_TABLE()

PreferenceDialog::PreferenceDialog():
        wxPropertySheetDialog()
{
}

PreferenceDialog::PreferenceDialog(wxWindow* parent,
                                wxWindowID id,
                                const wxString& title,
                                const wxPoint& pos,
                                const wxSize& size,
                                long style,
                                const wxString& name):
        wxPropertySheetDialog()
{
        Create(parent,id,title,pos,size,style,name);
}

PreferenceDialog::~PreferenceDialog() {
}

bool PreferenceDialog::Create(wxWindow* parent,
                                wxWindowID id,
                                const wxString& title,
                                const wxPoint& pos,
                                const wxSize& size,
                                long style,
                                const wxString& name)
{
        if (!wxPropertySheetDialog::Create(parent,id,title,pos,size,style,name))
                return false;
        
        CreateButtons(wxOK|wxCANCEL);
        
        wxPanel *panel;
        wxSize ctrlsize;
        
        // Appearance and Interface settings
        panel=new wxPanel(GetBookCtrl());
        _boxSelectionModus=new wxChoice(panel,wxID_ANY);
        _boxSelectionModus->Append(wxT("Select intersected objects"));
        _boxSelectionModus->Append(wxT("Select contained objects"));
        _mapBackgroundColourButton=new wxButton(panel,PreferenceDialog_ID_MapBackgroundColourButton);
        _selectionHandleColourButton=new wxButton(panel,PreferenceDialog_ID_SelectionHandleColourButton);
        _selectionMarkingColourButton=new wxButton(panel,PreferenceDialog_ID_SelectionMarkingColourButton);
        _aiArcWidth=new wxTextCtrl(panel,wxID_ANY);
        _aiNodeRadius=new wxTextCtrl(panel,wxID_ANY);

        wxFlexGridSizer* sizer_6=new wxFlexGridSizer(1,2,10,10);
        sizer_6->Add(new wxStaticText(panel,wxID_ANY,wxT("Box Selection Modus")));
        sizer_6->Add(_boxSelectionModus);
        sizer_6->Add(new wxStaticText(panel,wxID_ANY,wxT("Map Background Colour")));
        _mapBackgroundColourButton->SetMinSize(wxSize(30,30));
        _mapBackgroundColourButton->SetMaxSize(wxSize(30,30));
        sizer_6->Add(_mapBackgroundColourButton);
        sizer_6->Add(new wxStaticText(panel,wxID_ANY,wxT("Selection Handle Colour")));
        _selectionHandleColourButton->SetMinSize(wxSize(30,30));
        _selectionHandleColourButton->SetMaxSize(wxSize(30,30));
        sizer_6->Add(_selectionHandleColourButton);
        sizer_6->Add(new wxStaticText(panel,wxID_ANY,wxT("Selection Marking Colour")));
        _selectionMarkingColourButton->SetMinSize(wxSize(30,30));
        _selectionMarkingColourButton->SetMaxSize(wxSize(30,30));
        sizer_6->Add(_selectionMarkingColourButton);
        sizer_6->Add(new wxStaticText(panel,wxID_ANY,wxT("AI Arc Width (m)")));
        _aiArcWidth->SetMinSize(wxSize(30,30));
        _aiArcWidth->SetMaxSize(wxSize(30,30));
        sizer_6->Add(_aiArcWidth);
        sizer_6->Add(new wxStaticText(panel,wxID_ANY,wxT("AI Node Radius (m)")));
        _aiArcWidth->SetMinSize(wxSize(30,30));
        _aiArcWidth->SetMaxSize(wxSize(30,30));
        sizer_6->Add(_aiNodeRadius);
        
        panel->SetSizer(sizer_6);
        sizer_6->SetSizeHints(panel);

        GetBookCtrl()->AddPage(panel,wxT("Appearance & Interface"));
        
        // X-Plane import settings
        panel=new wxPanel(GetBookCtrl());

        _xplaneImportPath=new wxTextCtrl(panel,PreferenceDialog_ID_XPlaneImportPath);
        _xplaneImportPathBrowse=new wxButton(panel,PreferenceDialog_ID_XPlaneImportPathBrowse,wxT("..."));

        ctrlsize=_xplaneImportPath->GetSize();
        ctrlsize.SetWidth(400);
        _xplaneImportPath->SetMinSize(ctrlsize);

        wxBoxSizer *sizer_1=new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *sizer_2=new wxBoxSizer(wxHORIZONTAL);
        sizer_2->Add(_xplaneImportPath,wxSizerFlags(1).Expand());
        sizer_2->Add(_xplaneImportPathBrowse,wxSizerFlags(0).Centre());

        sizer_1->Add(new wxStaticText(panel,wxID_ANY,wxT("X-Plane database file")));
        sizer_1->Add(sizer_2);
        
        panel->SetSizer(sizer_1);
        sizer_1->SetSizeHints(panel);
        
        GetBookCtrl()->AddPage(panel,wxT("X-Plane"));
        
        // USGS image cache path + proxy settings
        panel=new wxPanel(GetBookCtrl());
        _usgsImagePath=new wxTextCtrl(panel,PreferenceDialog_ID_USGSCachePath);
        _usgsImagePathBrowse=new wxButton(panel,PreferenceDialog_ID_USGSCachePathBrowse,wxT("..."));

        wxBoxSizer *sizer_3=new wxBoxSizer(wxVERTICAL);
        sizer_3->Add(new wxStaticText(panel,wxID_ANY,wxT("USGS image cache path")));
        wxBoxSizer *sizer_4=new wxBoxSizer(wxHORIZONTAL);
        sizer_4->Add(_usgsImagePath,wxSizerFlags(1).Expand());
        sizer_4->Add(_usgsImagePathBrowse,wxSizerFlags(0).Centre());
        
        wxStaticBox *proxyGroup=new wxStaticBox(panel,wxID_ANY,wxT("Proxy options"));
        _useProxyCtrl=new wxCheckBox(panel,PreferenceDialog_ID_UseProxy,wxT("Use HTTP proxy"));
        _httpProxyNameLabel=new wxStaticText(panel,wxID_ANY,wxT("Proxy hostname"));
        _httpProxyName=new wxTextCtrl(panel,PreferenceDialog_ID_ProxyName);
        _httpProxyPortLabel=new wxStaticText(panel,wxID_ANY,wxT("Proxy port"));
        _httpProxyPort=new wxTextCtrl(panel,PreferenceDialog_ID_ProxyPort);

        ctrlsize=_usgsImagePath->GetSize();
        ctrlsize.SetWidth(400);
        _usgsImagePath->SetMinSize(ctrlsize);
        ctrlsize=_httpProxyName->GetSize();
        ctrlsize.SetWidth(300);
        _httpProxyName->SetMinSize(ctrlsize);
        ctrlsize=_httpProxyPort->GetSize();
        ctrlsize.SetWidth(150);
        _httpProxyPort->SetMinSize(ctrlsize);

        wxStaticBoxSizer* sizer_5=new wxStaticBoxSizer(proxyGroup,wxVERTICAL);
        sizer_5->Add(_useProxyCtrl);
        sizer_5->Add(_httpProxyNameLabel);
        sizer_5->Add(_httpProxyName);
        sizer_5->Add(_httpProxyPortLabel);
        sizer_5->Add(_httpProxyPort);
        
        sizer_3->Add(sizer_4);
        sizer_3->Add(sizer_5);
        panel->SetSizer(sizer_3);
        sizer_3->SetSizeHints(panel);
        
        GetBookCtrl()->AddPage(panel,wxT("USGS download"));
        
        // FlightGear Data Options
        panel=new wxPanel(GetBookCtrl());

        _fgfsDataPath=new wxTextCtrl(panel,PreferenceDialog_ID_FGFSDataPath);
        _fgfsDataPathBrowse=new wxButton(panel,PreferenceDialog_ID_FGFSDataPathBrowse,wxT("..."));

        ctrlsize=_fgfsDataPath->GetSize();
        ctrlsize.SetWidth(400);
        _fgfsDataPath->SetMinSize(ctrlsize);

        wxBoxSizer *sizer_7=new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *sizer_8=new wxBoxSizer(wxHORIZONTAL);
        sizer_8->Add(_fgfsDataPath,wxSizerFlags(1).Expand());
        sizer_8->Add(_fgfsDataPathBrowse,wxSizerFlags(0).Centre());

        sizer_7->Add(new wxStaticText(panel,wxID_ANY,wxT("FlightGear Data path")));
        //sizer_7->Add(sizer_8);
        
        //panel->SetSizer(sizer_7);
        //sizer_7->SetSizeHints(panel);
        // Try the same for the FlightGear Scenery path
        //wxBoxSizer *sizer_9=new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *sizer_9=new wxBoxSizer(wxHORIZONTAL);
        _fgfsSceneryPath=new wxTextCtrl(panel,PreferenceDialog_ID_FGFSSceneryPath);
        ctrlsize=_fgfsSceneryPath->GetSize();
        ctrlsize.SetWidth(400);
        _fgfsSceneryPath->SetMinSize(ctrlsize);
        _fgfsSceneryPathBrowse=new wxButton(panel,PreferenceDialog_ID_FGFSSceneryPathBrowse,wxT("..."));

        sizer_9->Add(_fgfsSceneryPath,wxSizerFlags(1).Expand());
        sizer_9->Add(_fgfsSceneryPathBrowse,wxSizerFlags(0).Centre());
        sizer_7->Add(sizer_8);
        sizer_7->Add(new wxStaticText(panel,wxID_ANY,wxT("FlightGear Scenery path")));
        sizer_7->Add(sizer_9);
        
        panel->SetSizer(sizer_7);
        sizer_7->SetSizeHints(panel);

        
        GetBookCtrl()->AddPage(panel,wxT("FlightGear"));
        
        LayoutDialog();
        return true;
}

bool PreferenceDialog::TransferDataToWindow() {
        if (!wxPropertySheetDialog::TransferDataToWindow())
                return false;
        
        _boxSelectionModus->SetSelection(SelectionMode::Include_Intersected?0:1);
        _mapBackgroundColourButton->SetBackgroundColour(MapEditor::Map_Background_Colour);
        _selectionHandleColourButton->SetBackgroundColour(MapSelection::Selection_Handle_Colour);
        _selectionMarkingColourButton->SetBackgroundColour(MapSelection::Selection_Marking_Colour);
        
#if wxHAS_TEXT_WINDOW_STREAM
        std::ostream oswidth(_aiArcWidth);
        oswidth << AINetworkArc::Arc_Width;
        
        std::ostream osradius(_aiNodeRadius);
        osradius << AINetworkNode::Radius;
#else
        std::ostringstream oss;
        oss << AINetworkArc::Arc_Width;
        _aiArcWidth->SetValue( oss.str() );
        oss.str( "" );
        oss << AINetworkNode::Radius;
        _aiNodeRadius->SetValue( oss.str() );
#endif
        
        wxConfigBase* config=wxConfigBase::Get();
        
        wxString xplaneImportPathStr;
        if (config->Read(wxT("xplane-data-path"),&xplaneImportPathStr)) {
                _xplaneImportPath->SetValue(xplaneImportPathStr);
        }
        
        wxString usgsCachePathString;
        if (config->Read(wxT("History/Paths/last-download-image-path"),&usgsCachePathString)) {
                _usgsImagePath->SetValue(usgsCachePathString);
        }
        
        bool useProxy=false;
        config->Read(wxT("Connection/UseHttpProxy"), &useProxy);
        _useProxyCtrl->SetValue(useProxy);
        
        wxString httpProxyNameString,httpProxyPortString;
        if (config->Read(wxT("Connection/HttpProxyName"),&httpProxyNameString)) {
                _httpProxyName->SetValue(httpProxyNameString);
        }
        if (config->Read(wxT("Connection/HttpProxyPort"),&httpProxyPortString)) {
                _httpProxyPort->SetValue(httpProxyPortString);
        }
        
        _httpProxyNameLabel->Enable(useProxy);
        _httpProxyName->Enable(useProxy);
        _httpProxyPortLabel->Enable(useProxy);
        _httpProxyPort->Enable(useProxy);
        
        wxString fgfsDataPathStr;
        if (config->Read(wxT("fgfs-data-path"),&fgfsDataPathStr)) {
                _fgfsDataPath->SetValue(fgfsDataPathStr);
        }

        wxString fgfsSceneryPathStr;
        if (config->Read(wxT("fgfs-scenery-path"),&fgfsSceneryPathStr)) {
                _fgfsSceneryPath->SetValue(fgfsSceneryPathStr);
        }

        return true;
}

bool PreferenceDialog::TransferDataFromWindow() {
        if (!wxPropertySheetDialog::TransferDataFromWindow())
                return false;
        
        SelectionMode::Include_Intersected=(_boxSelectionModus->GetSelection()==0);
        
        MapEditor::Map_Background_Colour=_mapBackgroundColourButton->GetBackgroundColour();
        MapSelection::Selection_Handle_Colour=_selectionHandleColourButton->GetBackgroundColour();
        MapSelection::Selection_Marking_Colour=_selectionMarkingColourButton->GetBackgroundColour();
        _aiArcWidth->GetValue().ToDouble(&AINetworkArc::Arc_Width);
        _aiNodeRadius->GetValue().ToDouble(&AINetworkNode::Radius);
        
        // refresh all editors
        const MapEditorList &editorList=MapEditor::GetEditorList();
        MapEditorList::Node *node;
        for (node=editorList.GetFirst();node;node=node->GetNext()) {
                node->GetData()->DamageAll();
        }

        wxConfigBase* config=wxConfigBase::Get();
        
        wxString xplaneImportPathStr=_xplaneImportPath->GetValue();
        if (!xplaneImportPathStr.IsEmpty() && !::wxFileExists(xplaneImportPathStr)) {
                ::wxMessageBox(wxT("The X-Plane import file you specified does not exist.")
                               wxT("Please either specify an existing file or leave the field empty."),
                               wxT("X-Plane import database does not exist"),
                               wxOK|wxICON_HAND);
                return false;
        }
        
        if (!::wxDirExists(_usgsImagePath->GetValue())) {
                ::wxMessageBox(wxT("The USGS image cache path you specified does not exist.")
                               wxT("Please specify an existing directory."),
                               wxT("USGS image cache path does not exist"),
                               wxOK|wxICON_HAND);
                return false;
        }
        
        if (!_fgfsDataPath->GetValue().IsEmpty()) {
                wxFileName fgfsDataDir;
                fgfsDataDir.AssignDir(_fgfsDataPath->GetValue());

                if (!fgfsDataDir.DirExists()) {
                        ::wxMessageBox(wxT("The FlightGear data directory you specified does not exist.")
                                       wxT("Please specify an existing directory or none."),
                                       wxT("FlightGear data directory does not exist"),
                                       wxOK|wxICON_HAND);
                        return false;
                }
                fgfsDataDir.SetFullName(wxT("version"));
                if (!fgfsDataDir.FileExists()) {
                        ::wxMessageBox(wxT("The FlightGear data directory you is not valid (version file not found).")
                                       wxT("Please specify a valid directory or none."),
                                       wxT("FlightGear data directory is not valid"),
                                       wxOK|wxICON_HAND);
                        return false;
                }
        }
        
        config->Write(wxT("xplane-data-path"),xplaneImportPathStr);
        config->Write(wxT("fgfs-data-path"),_fgfsDataPath->GetValue());
        config->Write(wxT("fgfs-scenery-path"),_fgfsSceneryPath->GetValue());
        config->Write(wxT("History/Paths/last-download-image-path"),_usgsImagePath->GetValue());
        config->Write(wxT("Connection/UseHttpProxy"),_useProxyCtrl->GetValue());
        config->Write(wxT("Connection/HttpProxyName"),_httpProxyName->GetValue());
        config->Write(wxT("Connection/HttpProxyPort"),_httpProxyPort->GetValue());

        return true;
}

void PreferenceDialog::OnColourBrowse(wxCommandEvent& evt) {
        wxButton *button;
        switch (evt.GetId()) {
        case PreferenceDialog_ID_MapBackgroundColourButton:
                button=_mapBackgroundColourButton;
                break;
        case PreferenceDialog_ID_SelectionHandleColourButton:
                button=_selectionHandleColourButton;
                break;
        case PreferenceDialog_ID_SelectionMarkingColourButton:
                button=_selectionMarkingColourButton;
                break;
        }

        wxColour newcol=::wxGetColourFromUser(this,button->GetBackgroundColour());
        if (newcol.Ok()) {
                button->SetBackgroundColour(newcol);
        }
}

void PreferenceDialog::OnUseProxy(wxCommandEvent&) {
        bool useProxy=_useProxyCtrl->GetValue();
        _httpProxyNameLabel->Enable(useProxy);
        _httpProxyName->Enable(useProxy);
        _httpProxyPortLabel->Enable(useProxy);
        _httpProxyPort->Enable(useProxy);
}

void PreferenceDialog::OnXPlanePathBrowse(wxCommandEvent&) {
        wxString defFile=_xplaneImportPath->GetValue();
        wxFileName defFilename=wxFileName(defFile);
        
        const wxString& file = ::wxFileSelector(wxT("Choose X-Plane database file"),
                defFilename.GetPath(),
                defFilename.GetFullName(),
                wxT("dat"),
                wxT("X-Plane database file (*.dat)|*.dat|All files|*.*"),wxOPEN|wxFILE_MUST_EXIST);
        
        if (file.IsEmpty())
                return;
        
        _xplaneImportPath->SetValue(file);
}

void PreferenceDialog::OnUSGSImagePathBrowse(wxCommandEvent&) {
        wxString defPath=_usgsImagePath->GetValue();
        
        const wxString& dir = ::wxDirSelector(wxT("Choose USGS image cache path"),defPath);
        
        if (dir.IsEmpty())
                return;
        
        _usgsImagePath->SetValue(dir);
}

void PreferenceDialog::OnFGFSPathBrowse(wxCommandEvent&) {
        wxString defPath=_fgfsDataPath->GetValue();
        
        const wxString& dir = ::wxDirSelector(wxT("Choose FlightGear data directory"),defPath);
        
        if (dir.IsEmpty())
                return;
        
        _fgfsDataPath->SetValue(dir);
}

void PreferenceDialog::OnFGFSSceneryBrowse(wxCommandEvent&) {
        wxString defPath=_fgfsDataPath->GetValue();
        
        const wxString& dir = ::wxDirSelector(wxT("Choose FlightGear Scenery/Airport directory"),defPath);
        
        if (dir.IsEmpty())
                return;
        
        _fgfsSceneryPath->SetValue(dir);
}


