#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapSelectionActionsHandler.h"

#include "MapEditor.h"
#include "MapSelectionManager.h"
#include "MapObject.h"

IMPLEMENT_CLASS(MapSelectionActionsHandler,wxMenu);

MapSelectionActionsHandler MapSelectionActionsHandler::_instance;

MapSelectionActionsHandler::MapSelectionActionsHandler()
{
}

MapSelectionActionsHandler::~MapSelectionActionsHandler() {
}

void MapSelectionActionsHandler::OnBringToFront(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList::Node* node;
        for (node=selmgr->GetSelections().GetFirst();node;node=node->GetNext()) {
                MapObject* content=node->GetData()->GetContent();
                wxASSERT_MSG(content->GetContainer()!=NULL,
                     wxT("Cannot move an object that is not attached to a container"));
                content->GetContainer()->MoveObjectToTop(content);
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Changing object order"));
}

void MapSelectionActionsHandler::OnBringForward(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList::Node* node;
        for (node=selmgr->GetSelections().GetFirst();node;node=node->GetNext()) {
                MapObject* content=node->GetData()->GetContent();
                wxASSERT_MSG(content->GetContainer()!=NULL,
                     wxT("Cannot move an object that is not attached to a container"));
                content->GetContainer()->MoveObjectUp(content);
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Changing object order"));
}

void MapSelectionActionsHandler::OnBringBackward(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList::Node* node;
        for (node=selmgr->GetSelections().GetFirst();node;node=node->GetNext()) {
                MapObject* content=node->GetData()->GetContent();
                wxASSERT_MSG(content->GetContainer()!=NULL,
                     wxT("Cannot move an object that is not attached to a container"));
                content->GetContainer()->MoveObjectDown(content);
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Changing object order"));
}

void MapSelectionActionsHandler::OnBringToBack(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList::Node* node;
        for (node=selmgr->GetSelections().GetFirst();node;node=node->GetNext()) {
                MapObject* content=node->GetData()->GetContent();
                wxASSERT_MSG(content->GetContainer()!=NULL,
                     wxT("Cannot move an object that is not attached to a container"));
                content->GetContainer()->MoveObjectToBottom(content);
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Changing object order"));
}

void MapSelectionActionsHandler::OnDelete(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager &mgr=editor->GetSelectionManager();
        MapSelectionList selections=mgr.GetSelections();
        MapSelectionList::Node *node;
        for (node=selections.GetFirst();node;node=node->GetNext()) {
                MapObject* obj=node->GetData()->GetContent();
                obj->RemoveFromContainer();
        }
        
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Deleting objects"));
}

