#ifndef _UTM_H
#define _UTM_H

#include <Map/MapDatumProjectionDescriptor.h>
#include "datum.h"

#define REDFEARN_A0(d) (1.0 - (d.e_2 / 4.0) - (3.0 * d.e_2 * d.e_2 / 64.0) - (5.0 * d.e_2 * d.e_2 * d.e_2 / 256.0))
#define REDFEARN_A2(d) (3.0 * (d.e_2 + d.e_2 * d.e_2 / 4.0 + 15.0 * d.e_2 * d.e_2 * d.e_2 / 128.0) / 8.0)
#define REDFEARN_A4(d) (15.0 / 256.0 * (d.e_2 * d.e_2 + 3.0 * d.e_2 * d.e_2 * d.e_2 / 4.0))
#define REDFEARN_A6(d) (35.0 * d.e_2 * d.e_2 * d.e_2 / 3072.0)

/*
 * To convert to/from UTM coordinates we need to know the scale factor on the central meridian.
 * For UTM, this is always 0.9996.
 */
#define UTM_K0 (0.9996)			// UTM Scale factor on the central meridian

struct datum {
        datum(MapDatumProjectionDescriptor::Datum d):
                a(d.a),
                b(d.b),
                e_2(d.e_2),
                f_inv(d.f_inv),
                k0(UTM_K0),
                a0(REDFEARN_A0(d)),
                a2(REDFEARN_A2(d)),
                a4(REDFEARN_A4(d)),
                a6(REDFEARN_A6(d))
        {
        }
        
        double a,b,e_2,f_inv;
        double k0;
        double a0,a2,a4,a6;
};

// DCL - force zone forces the zone supplied to be used - this is useful for cases where an airport straddles a zone
long redfearn(struct datum *datum, double *utm_x, double *utm_y, 
              long *zone, double latitude, double longitude, long east_west, bool force_zone = false);
			  
long redfearn_inverse(struct datum *datum, double utm_x, double utm_y, 
                      long zone, double *latitude, double *longitude);
					  
#endif
