//*** MenuToolbar

//

// The implementation of the wxMenuButton and the wxMenuToolBar

//



//*** The simple Menu Button

//

// A simple pull-down button that pops-up a menu

//

// Can be used in a toolbar or all by it's lonesome in a dialog or window.



#include "wx/wxprec.h"





#ifdef __BORLANDC__

    #pragma hdrstop

#endif



// for all others, include the necessary headers (this file is usually all you

// need because it includes almost all "standard" wxWidgets headers)

#ifndef WX_PRECOMP

    #include "wx/wx.h"

#endif



#include "menutoolbar.h"

#include "wx/listimpl.cpp"

WX_DEFINE_LIST(wxMenuToolList);



/* XPM */

static char *pulldown_xpm[] = {

/* columns rows colors chars-per-pixel */

"16 15 2 1",

". c Black",

"  c None",

/* pixels */

"                ",

"                ",

"                ",

"                ",

"                ",

"                ",

"      .....     ",

"       ...      ",

"        .       ",

"                ",

"                ",

"                ",

"                ",

"                ",

"                "

};





DEFINE_EVENT_TYPE(wxEVT_FILL_MENU)





/**************************/

/*** THE WX-MENU BUTTON ***/

/**************************/



/******************************************/

/*** Creation, Destruction, and Cleanup ***/

/******************************************/



void 

wxMenuButton::Create(wxWindow *window, wxToolBar *toolBar, int ID)

{

  wxBitmap bmpPulldown(pulldown_xpm);

  wxSize pulldownSize;



#ifdef __WXMSW__

  pulldownSize.Set(12,15);

#else

  pulldownSize.Set(16,15);

#endif



  m_toolBar = toolBar;

  m_menu = NULL;



  ((wxBitmapButton *)this)->Create(window, ID, bmpPulldown, wxDefaultPosition, 

         pulldownSize, wxNO_BORDER);

  Connect(ID, wxEVT_LEFT_DOWN, wxMouseEventHandler(wxMenuButton::DoProcessLeftClick) );

}





wxMenuButton::~wxMenuButton()

{

  if(m_menu) delete m_menu;

}





/********************************/

/* wxMenuButton::DoProcessClick */

/********************************/

void 

wxMenuButton::DoProcessLeftClick(wxMouseEvent& event)

{

  wxPoint menu_pos;



  if(!m_menu) {

    m_menu = new wxMenu;

  }



  // *** Give others the opportunity to create and/or change the menu

  wxCommandEvent eventFill( wxEVT_FILL_MENU, GetId() );

  eventFill.SetClientData( m_menu );

  GetEventHandler()->ProcessEvent( eventFill );



  if(m_toolBar) {

    wxSize tool_size = m_toolBar->GetToolSize();

    wxSize button_size = GetSize();



    // ** Assume that pulldown is to the right of a standard toolbar button,

    //    so, move the x position back one standard toolbar button's width

    menu_pos.x = - tool_size.GetWidth();

    menu_pos.y =  button_size.GetHeight()/2 + tool_size.GetHeight()/2;



#if defined(__WXMAC__)

	wxSize tbar_size = m_toolBar->GetSize();

    wxPoint button_pos = GetPosition();

	int iToolSep = m_toolBar->GetToolSeparation();

	if(iToolSep == 0) iToolSep = 5;



    menu_pos.x += - iToolSep;

    menu_pos.y = tbar_size.GetHeight() - button_pos.y/2;



#endif

  }

  else {

    wxSize button_size;

    button_size = GetSize();

    menu_pos.x = 0;

    menu_pos.y = button_size.GetHeight();

  }



  PopupMenu(m_menu, menu_pos);

}







/************************/

/*** THE WX-MENU-TOOL ***/

/************************/



wxMenuTool::wxMenuTool(wxToolBarToolBase *new_tool, int toolId)

{

  m_tool = new_tool;

  m_menu = NULL;

  m_toolId = toolId;

}



wxMenuTool::~wxMenuTool()

{

  if(m_menu)  delete m_menu;

}







/****************************/

/*** THE WX-MENU TOOL BAR ***/

/****************************/



/****************************/

/* Creation and Destruction */

/****************************/



wxMenuToolBar::wxMenuToolBar(wxFrame* parent, 

                      wxWindowID id, 

                      const wxPoint& pos, 

                      const wxSize& size, 

                      long style, 

                      const wxString& name)



  : wxToolBar(parent, id, pos, size, style, name)

{

  m_frame = parent;

  m_menuTools = NULL;



  Connect(id, wxEVT_LEFT_DOWN, wxMouseEventHandler(wxMenuToolBar::DoProcessLeftClick) );

}





wxMenuToolBar::~wxMenuToolBar()

{

  if(m_menuTools)  delete m_menuTools;

}





/****************/

/* Adding Tools */

/****************/



void

wxMenuToolBar::NewMenuTool(wxMenuTool *menu_tool)

{

  if(m_menuTools == NULL) {

    m_menuTools = new wxMenuToolList();

    m_menuTools->DeleteContents(true);

  }



  m_menuTools->Append(menu_tool);

}





void

wxMenuToolBar::AddMenuButtonTool(int toolId, 

                         const wxString& label, 

                         const wxBitmap& bitmap1, 

                         const wxString& shortHelpString)

{

#ifdef __WXMSW__

  wxToolBarToolBase *tb_base;

  tb_base = AddTool(toolId, label, bitmap1, shortHelpString, wxITEM_NORMAL);

  NewMenuTool(new wxMenuTool(tb_base, toolId));

#elif defined(__WXMAC__)

  wxSize tool_size = GetToolSize();

  wxMacMenuTool *menu_button = new wxMacMenuTool((wxMenuToolBar *)this, toolId, bitmap1, 

                       wxDefaultPosition, tool_size, (long int)wxNO_BORDER);

  AddControl(menu_button);

#endif

}





wxMenuButton *

wxMenuToolBar::AddMenuPulldownTool(int toolId, 

                         const wxString& label, 

                         const wxString& shortHelpString)

{

  wxMenuButton *menu_button = new wxMenuButton(this, toolId);

  AddControl(menu_button);



  return menu_button;

}





void 

wxMenuToolBar::DoProcessLeftClick(wxMouseEvent& event)

{

  wxMenuToolList::Node *node = NULL;

  wxMenuTool *menu_tool = NULL;

  wxToolBarToolBase *clickTool = FindToolForPosition(event.m_x, event.m_y);



  if(clickTool == NULL || m_menuTools == NULL) {  event.Skip();  return;  }



  // search for clickTool in the list of menu tools

  node = m_menuTools->GetFirst();

  for( ; node ; node = node->GetNext()) {

    menu_tool = node->GetData();

    if(menu_tool->m_tool == clickTool) break;

  }

  if(node == NULL) {  event.Skip();  return;  }



  // *** Give others the opportunity to create and/or change the menu

  wxCommandEvent eventFill( wxEVT_FILL_MENU, menu_tool->m_toolId );

  if(menu_tool->m_menu == NULL)  menu_tool->m_menu = new wxMenu;

  eventFill.SetClientData( menu_tool->m_menu );

  GetEventHandler()->ProcessEvent( eventFill );



  wxSize tbar_size = GetSize();

  wxPoint menu_pos;



  menu_pos.x = event.m_x - 20;

  menu_pos.y = tbar_size.GetHeight() - 2;



  PopupMenu(menu_tool->m_menu, menu_pos);

}





#ifdef __WXMAC__



//

// *** Macintosh Implementation

//



wxMacMenuTool::wxMacMenuTool(wxMenuToolBar* parent, 

                   wxWindowID id, 

                   const wxBitmap &bmp, 

                   const wxPoint& pos, 

                   const wxSize& size, 

                   long style) : wxBitmapButton(parent, id, bmp, pos, size, style)

{

  m_toolBar = parent;

  m_menu = NULL;

  Connect(id, wxEVT_LEFT_DOWN, wxMouseEventHandler(wxMacMenuTool::DoProcessLeftClick) );

}



wxMacMenuTool::~wxMacMenuTool()

{

  if(m_menu)  delete m_menu;

}



void 

wxMacMenuTool::DoProcessLeftClick(wxMouseEvent& event)

{

  wxPoint menu_pos;



  if(!m_menu) {

    m_menu = new wxMenu;

  }



  // *** Give others the opportunity to create and/or change the menu

  wxCommandEvent eventFill( wxEVT_FILL_MENU, GetId() );

  eventFill.SetClientData( m_menu );

  GetEventHandler()->ProcessEvent( eventFill );



  wxSize tool_size = m_toolBar->GetToolSize();

  wxSize tbar_size = m_toolBar->GetSize();



  menu_pos.x = event.m_x - 20;  // Just move it back a nice amount from the mouse click position

  menu_pos.y =  tbar_size.GetHeight() - 1;



  PopupMenu(m_menu, menu_pos);

}



#endif

